clear; clf; clc;
set(gca,'FontSize', 16)

% Initialize sss problem
sigma = 10^(-3)
ratio = 1
eps = 10^(-12);
[A, b] = testsystem(sigma, ratio);

n = size(A,1);
x_0 = zeros(n,1);
x_direct = A\b;

% solve by ssslanczos

S = A - sigma*eye(n);
[x,j,res_norm,res_hist] = mrs3(sigma,S,x_0,b,eps,1000);

S = A - sigma*eye(n);
[x,j,res_norm,res_hist] = mrs3(sigma,S,x_0,b,eps,1000);

disp('mrs3 norm of residual')
niter = size(res_hist,1);
err_norm = norm(x-x_direct)
%disp([niter norm(b-A*x) err_norm(niter)])

h1 = semilogy([1:niter]-1, res_hist,'k');
set(h1,'LineWidth',2);
hold on

% fill preconditioner with the identity

L = sparse(eye(n));
U = L;

% solution by GMRES
%[x,res_norm,err_norm]=gmres(L,U,A,b,eps);
%disp('GMRES norm of residual')
%niter = size(res_norm,2);
%disp([niter norm(b-A*x) err_norm])
%semilogy(res_norm,'g');

% solution by restarted GMRES

nrestart = 3;
[x,res_norm,err_norm]=gmresrestart(L,U,A,b,eps,nrestart);

disp(['GMRES(', num2str(nrestart), ') norm of residual'])
res_norm = res_norm(1:min(size(res_norm,2),800));
niter = size(res_norm,2);
disp([niter norm(b-A*x) err_norm])

h2 = semilogy([1:niter]-1, res_norm,'k:');

% solution by Bi-CGSTAB

[x,error,res_norm,flag] = bicgstab(A,b,L,400,eps);
%flag

disp('Bi-CGSTAB norm of residual')
niter = size(res_norm,2);
disp([niter norm(b-A*x) error])

h4 = semilogy(2*[1:niter]-2, res_norm,'k--');

xlabel('matrix vector products')
ylabel('||b - A x ||')
legend([h1,h2,h4], 'MRS^3',['GMRES(', num2str(nrestart), ')'], 'Bi-CGSTAB');
%axis([0 400 1e-12 1e0])
axis([0 600 1e-12 1e6])
print -dpdf figure.pdf
print -deps figure.eps
